//-----------------------------------------------------------------------------
// MK3_2560_EEPROM
// Version 1.1
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// EEPROM Verwaltung (vom internen EEprom der CPU)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_EEPROM.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : nichts
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_EEPROM_BYTE 1  // Byte-Werte lesen/schreiben
#define USE_MK3_EEPROM_INT 1   // Integer-Werte lesen/schreiben

//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_EEPROM" starten
// 02. Funktion "MK3_EEPROM_WR8" schreibt einen Byte-Wert ins EEprom
// 03. Funktion "MK3_EEPROM_RD8" liest einen Byte-Wert aus dem EEprom
// 04. Funktion "MK3_EEPROM_WR16" schreibt einen Integer-Wert ins EEprom
// 05. Funktion "MK3_EEPROM_RD16" liest einen Integer-Wert aus dem EEprom
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// keine


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_EEPROM(void);
void MK3_EEPROM_WR8(unsigned int adr, unsigned char wert);	// zum schreiben eines Bytes (8Bit)
unsigned char MK3_EEPROM_RD8(unsigned int adr);			// zum lesen eines Bytes (8Bit)
void MK3_EEPROM_WR16(unsigned int adr, unsigned int wert);	// zum schreiben eines INT (16Bit)
unsigned int MK3_EEPROM_RD16(unsigned int adr);			// zum lesen eines INT (16Bit)
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// EEProm Groesse
//-----------------------------------------------------------------------------
#define MAX_EE_ADR 0xFFF  // max Adresse vom EEprom (4kByte)



//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
void EEPROM_WR(unsigned int adr, unsigned char wert);
unsigned char EEPROM_RD(unsigned int adr);



//-----------------------------------------------------------------------------
void MK3_INIT_EEPROM(void) {
  // Init vom EEprom

  EECR=0x00;  // init vom EECR-Register

  // Adresse auf 0 setzen
  // damit bei einem Fehler nur diese Adresse
  // beschaedigt wird
  EEAR = 0x00;
}




//-----------------------------------------------------------------------------
#if USE_MK3_EEPROM_BYTE==1


//-----------------------------------------------------------------------------
void MK3_EEPROM_WR8(unsigned int adr, unsigned char wert) {
  // schreibt einen 8Bit-Wert ins EEprom

  if(adr<=MAX_EE_ADR) {
    // wenn Adresse gueltig
    EEPROM_WR(adr,wert);
  }
}

//-----------------------------------------------------------------------------
unsigned char MK3_EEPROM_RD8(unsigned int adr) {
  // liesst einen 8Bit-Wert aus dem EEprom
  unsigned char ret_wert=0;

  if(adr<=MAX_EE_ADR) {
    // wenn Adresse gueltig
    ret_wert=EEPROM_RD(adr);
  }

  return(ret_wert);
}

#endif
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
#if USE_MK3_EEPROM_INT==1


//-----------------------------------------------------------------------------
void MK3_EEPROM_WR16(unsigned int adr, unsigned int wert) {
  // schreibt einen 16Bit-Wert ins EEprom
  unsigned char lo,hi;

  if(adr<=MAX_EE_ADR-1) {
    // wenn Adresse gueltig
    lo=(wert&0x00FF);
    hi=(wert>>8);
    EEPROM_WR(adr+0,lo);
    EEPROM_WR(adr+1,hi);
  }
}


//-----------------------------------------------------------------------------
unsigned int MK3_EEPROM_RD16(unsigned int adr) {
  // liesst einen 16Bit-Wert aus dem EEprom
  unsigned int ret_wert=0;
  unsigned char lo,hi;

  if(adr<=MAX_EE_ADR-1) {
    // wenn Adresse gueltig
    lo=EEPROM_RD(adr+0);
    hi=EEPROM_RD(adr+1);
    ret_wert=(hi<<8);
    ret_wert=ret_wert+lo;
  }  

  return(ret_wert);
}


#endif
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
void EEPROM_WR(unsigned int adr, unsigned char wert) {
  // interne write Funktion

  //--------------------------------------------------
  // VORSICHT !!
  // mit dem myAVR-Workpad funktioniert
  // der C-Code fuer das schreiben
  // in das EEprom nicht (Timing probleme)
  // 
  // -> zwei Zeilen mussten aus dem
  //    Grund in Assembler programiert werden
  //--------------------------------------------------
  // Adresse von "EECR"  = 0x1F
  // Bit-Adr von "EEMPE" = 2
  // Bit-Adr von "EEPE"  = 1
  //--------------------------------------------------

  while(EECR & (1<<EEPE));
  EEAR = adr;
  EEDR = wert;
  //  EECR |= (1<<EEMPE); // diese zwei Zeilen
  //  EECR |= (1<<EEPE);  // funktionieren nicht

  __asm volatile  (       // und muessen durch
     "sbi 0x1F,2" "\r\n"  // assember code
     "sbi 0x1F,1" "\r\n"  // ersetzt werden
  );

  // Adresse wieder auf 0 setzen
  // damit bei einem Fehler nur diese Adresse
  // beschaedigt wird
  EEAR = 0x00;
}

//-----------------------------------------------------------------------------
unsigned char EEPROM_RD(unsigned int adr) {
  // interne read Funktion
  unsigned char ret_wert=0;

  while(EECR & (1<<EEPE));
  EEAR = adr;
  EECR |= (1<<EERE);
  ret_wert=EEDR;

  // Adresse wieder auf 0 setzen
  // damit bei einem Fehler nur diese Adresse
  // beschaedigt wird
  EEAR = 0x00;
  
  return(ret_wert);
}


